@@ -5,6 +5,7 @@ mutable struct LSR1Data{T, I <: Integer}
   mem::I
   scaling::Bool
   scaling_factor::T
+  opnorm_upper_bound::T # Upper bound for the operator norm ‖Bₖ‖₂ ≤ ‖B₀‖₂ + ∑ᵢ |σᵢ|‖aᵢ‖₂².
   s::Vector{Vector{T}}
   y::Vector{Vector{T}}
   ys::Vector{T}
@@ -20,6 +21,7 @@ function LSR1Data(T::Type, n::I; mem::I = 5, scaling::Bool = true) where {I <: I
     max(mem, 1),
     scaling,
     convert(T, 1),
+    convert(T, 1),
     [zeros(T, n) for _ = 1:mem],
     [zeros(T, n) for _ = 1:mem],
     zeros(T, mem),
@@ -136,7 +138,7 @@ function push!(op::LSR1Operator, s::AbstractVector, y::AbstractVector)
     sufficient_curvature = abs(ys) ≥ ϵ * yNorm * sNorm
     if sufficient_curvature
       scaling_factor = ys / yy
-      @. data.tmp = y - s / scaling_factor
+      data.tmp .= y .- s ./ scaling_factor
       scaling_condition = norm(data.tmp) >= ϵ * yNorm * sNorm
     end
   end
@@ -152,7 +154,11 @@ function push!(op::LSR1Operator, s::AbstractVector, y::AbstractVector)
   data.ys[data.insert] = ys
 
   # update scaling factor
-  data.scaling && (data.scaling_factor = ys / yy)
+  data.opnorm_upper_bound = convert(typeof(data.opnorm_upper_bound), 1)
+  if data.scaling
+    (data.scaling_factor = ys / yy)
+    !iszero(data.scaling_factor) && (data.opnorm_upper_bound = 1 / abs(op.data.scaling_factor))
+  end
 
   # update next insertion position
   data.insert = mod(data.insert, data.mem) + 1
@@ -161,15 +167,17 @@ function push!(op::LSR1Operator, s::AbstractVector, y::AbstractVector)
   for i = 1:(data.mem)
     k = mod(data.insert + i - 2, data.mem) + 1
     if data.ys[k] != 0
-      data.a[k] .= data.y[k] - data.s[k] / data.scaling_factor  # = y - B₀ * s
+      data.a[k] .= data.y[k] .- data.s[k] ./ data.scaling_factor  # = y - B₀ * s
       for j = 1:(i - 1)
         l = mod(data.insert + j - 2, data.mem) + 1
         if data.ys[l] != 0
           as = dot(data.a[l], data.s[k]) / data.as[l]
-          data.a[k] .-= as * data.a[l]
+          data.a[k] .-= as .* data.a[l]
         end
       end
       data.as[k] = dot(data.a[k], data.s[k])
+
+      !iszero(data.as[k]) && (data.opnorm_upper_bound += norm(data.a[k])^2/abs(data.as[k]))
     end
   end
 
